@@ -182,6 +182,16 @@ def karyogram(bp, sample, out, title, centromeres, colors, verbosity):
         " flag does not work when your output file is in PGEN format."
     ),
 )
+@click.option(
+    "--no_replacement",
+    is_flag=True,
+    required=False,
+    default=False,
+    help=(
+        "Flag used to determine whether to sample reference haplotypes"
+        " with or without replacement. (Default = Replacement)"
+    ),
+)
 @click.option(
     "--only_breakpoint",
     is_flag=True,
@@ -211,6 +221,7 @@ def simgenotype(
     region,
     pop_field,
     sample_field,
+    no_replacement,
     only_breakpoint,
     verbosity,
 ):
@@ -264,7 +275,15 @@ def simgenotype(
 
     # simulate breakpoints
     popsize = validate_params(
-        model, mapdir, chroms, popsize, ref_vcf, sample_info, region, only_breakpoint
+        model,
+        mapdir,
+        chroms,
+        popsize,
+        ref_vcf,
+        sample_info,
+        no_replacement,
+        region,
+        only_breakpoint,
     )
     samples, pop_dict, breakpoints = simulate_gt(
         model, mapdir, chroms, region, popsize, log, seed
@@ -284,6 +303,7 @@ def simgenotype(
             region,
             pop_field,
             sample_field,
+            no_replacement,
             out,
             log,
         )
@@ -591,7 +611,7 @@ def transform(
     discard_missing: bool = False,
     ancestry: bool = False,
     output: Path = Path("-"),
-    verbosity: str = "CRITICAL",
+    verbosity: str = "INFO",
 ):
     """
     Creates a VCF composed of haplotypes
@@ -755,7 +775,7 @@ def ld(
     discard_missing: bool = False,
     from_gts: bool = False,
     output: Path = Path("/dev/stdout"),
-    verbosity: str = "CRITICAL",
+    verbosity: str = "INFO",
 ):
     """
     Compute the pair-wise LD (Pearson's correlation) between haplotypes (or variants)
@@ -841,7 +861,7 @@ def index(
     haplotypes: Path,
     sort: bool = False,
     output: Path = None,
-    verbosity: str = "CRITICAL",
+    verbosity: str = "INFO",
 ):
     """
     Takes in an unsorted .hap file and outputs it as a .gz and a .tbi file
