@@ -3,4 +3,4 @@ from .phenotypes import Phenotypes
 from .covariates import Covariates
 from .breakpoints import Breakpoints, HapBlock
 from .haplotypes import Extra, Variant, Haplotype, Haplotypes
-from .genotypes import Genotypes, GenotypesRefAlt, GenotypesPLINK
+from .genotypes import Genotypes, GenotypesVCF, GenotypesPLINK
