@@ -11,7 +11,7 @@ import numpy.typing as npt
 from pysam import TabixFile
 
 from .data import Data
-from .genotypes import GenotypesRefAlt
+from .genotypes import GenotypesVCF
 
 
 @dataclass
@@ -453,7 +453,7 @@ class Haplotype:
         """
         return tuple(extra.name for extra in cls._extras)
 
-    def transform(self, genotypes: GenotypesRefAlt) -> npt.NDArray[bool]:
+    def transform(self, genotypes: GenotypesVCF) -> npt.NDArray[bool]:
         """
         Transform a genotypes matrix via the current haplotype
 
@@ -462,7 +462,7 @@ class Haplotype:
 
         Parameters
         ----------
-        genotypes : GenotypesRefAlt
+        genotypes : GenotypesVCF
             The genotypes which to transform using the current haplotype
 
             If the genotypes have not been loaded into the Genotypes object yet, this
@@ -490,7 +490,7 @@ class Haplotype:
         allele_arr = np.array(
             [
                 [
-                    [int(var.allele != gts.variants[i]["ref"])]
+                    [int(var.allele != gts.variants[i]["alleles"][0])]
                     for i, var in enumerate(self.variants)
                 ]
             ]
@@ -1053,9 +1053,9 @@ class Haplotypes(Data):
 
     def transform(
         self,
-        gts: GenotypesRefAlt,
-        hap_gts: GenotypesRefAlt = None,
-    ) -> GenotypesRefAlt:
+        gts: GenotypesVCF,
+        hap_gts: GenotypesVCF = None,
+    ) -> GenotypesVCF:
         """
         Transform a genotypes matrix via the current haplotype
 
@@ -1064,23 +1064,23 @@ class Haplotypes(Data):
 
         Parameters
         ----------
-        gts : GenotypesRefAlt
+        gts : GenotypesVCF
             The genotypes which to transform using the current haplotype
-        hap_gts: GenotypesRefAlt
-            An empty GenotypesRefAlt object into which the haplotype genotypes should
+        hap_gts: GenotypesVCF
+            An empty GenotypesVCF object into which the haplotype genotypes should
             be stored
 
         Returns
         -------
-        GenotypesRefAlt
+        GenotypesVCF
             A Genotypes object composed of haplotypes instead of regular variants.
         """
-        # Initialize GenotypesRefAlt return value
+        # Initialize GenotypesVCF return value
         if hap_gts is None:
-            hap_gts = GenotypesRefAlt(fname=None, log=self.log)
+            hap_gts = GenotypesVCF(fname=None, log=self.log)
         hap_gts.samples = gts.samples
         hap_gts.variants = np.array(
-            [(hap.id, hap.chrom, hap.start, "A", "T") for hap in self.data.values()],
+            [(hap.id, hap.chrom, hap.start, ("A", "T")) for hap in self.data.values()],
             dtype=hap_gts.variants.dtype,
         )
         # build a fast data structure for querying the alleles in each haplotype:
@@ -1104,7 +1104,7 @@ class Haplotypes(Data):
         # with shape (1, gts.data.shape[1], 1) for broadcasting later
         allele_arr = np.array(
             [
-                int(allele != gts.variants[i]["ref"])
+                int(allele != gts.variants[i]["alleles"][0])
                 for i, (vID, allele) in enumerate(alleles)
             ],
             dtype=gts.data.dtype,
