@@ -28,7 +28,7 @@ class HaplotypeAncestry(data.Haplotype):
         default=(data.Extra("ancestry", "s", "Local ancestry"),),
     )
 
-    def transform(self, genotypes: data.GenotypesRefAlt) -> npt.NDArray[bool]:
+    def transform(self, genotypes: data.GenotypesVCF) -> npt.NDArray[bool]:
         """
         Transform a genotypes matrix via the current haplotype and its ancestral
         population
@@ -51,7 +51,7 @@ class HaplotypeAncestry(data.Haplotype):
         allele_arr = np.array(
             [
                 [
-                    [int(var.allele != gts.variants[i]["ref"])]
+                    [int(var.allele != gts.variants[i]["alleles"][0])]
                     for i, var in enumerate(self.variants)
                 ]
             ]
@@ -91,14 +91,14 @@ class HaplotypesAncestry(data.Haplotypes):
     def transform(
         self,
         gts: data.GenotypesAncestry,
-        hap_gts: data.GenotypesRefAlt = None,
-    ) -> data.GenotypesRefAlt:
-        # Initialize GenotypesRefAlt return value
+        hap_gts: data.GenotypesVCF = None,
+    ) -> data.GenotypesVCF:
+        # Initialize GenotypesVCF return value
         if hap_gts is None:
-            hap_gts = data.GenotypesRefAlt(fname=None, log=self.log)
+            hap_gts = data.GenotypesVCF(fname=None, log=self.log)
         hap_gts.samples = gts.samples
         hap_gts.variants = np.array(
-            [(hap.id, hap.chrom, hap.start, "A", "T") for hap in self.data.values()],
+            [(hap.id, hap.chrom, hap.start, ("A", "T")) for hap in self.data.values()],
             dtype=hap_gts.variants.dtype,
         )
         # build a fast data structure for querying the alleles in each haplotype:
@@ -125,7 +125,7 @@ class HaplotypesAncestry(data.Haplotypes):
         # with shape (1, gts.data.shape[1], 1) for broadcasting later
         allele_arr = np.array(
             [
-                int(allele != gts.variants[i]["ref"])
+                int(allele != gts.variants[i]["alleles"][0])
                 for i, (vID, allele) in enumerate(alleles)
             ],
             dtype=gts.data.dtype,
@@ -147,9 +147,9 @@ class HaplotypesAncestry(data.Haplotypes):
         return hap_gts
 
 
-class GenotypesAncestry(data.GenotypesRefAlt):
+class GenotypesAncestry(data.GenotypesVCF):
     """
-    Extends the GenotypesRefAlt class for ancestry data
+    Extends the GenotypesVCF class for ancestry data
 
     The ancestry information is stored within the FORMAT field of the VCF
 
@@ -162,7 +162,7 @@ class GenotypesAncestry(data.GenotypesRefAlt):
     samples : tuple[str]
         See documentation for :py:attr:`~.Genotypes.samples`
     variants : np.array
-        See documentation for :py:attr:`~.GenotypesRefAlt.variants`
+        See documentation for :py:attr:`~.GenotypesVCF.variants`
     valid_labels: np.array
         Reference VCF sample and respective variant grabbed for
         each sample.
@@ -479,13 +479,14 @@ class GenotypesAncestry(data.GenotypesRefAlt):
             for sample in self.samples:
                 vcf.header.add_sample(sample)
         self.log.info("Writing VCF records")
+        phased = self._prephased or (self.data.shape[2] < 3)
         for var_idx, var in enumerate(self.variants):
             rec = {
                 "contig": var["chrom"],
                 "start": var["pos"],
-                "stop": var["pos"] + len(var["ref"]) - 1,
+                "stop": var["pos"] + len(var["alleles"][0]) - 1,
                 "qual": None,
-                "alleles": tuple(var[["ref", "alt"]]),
+                "alleles": var["alleles"],
                 "id": var["id"],
                 "filter": None,
             }
@@ -507,8 +508,11 @@ class GenotypesAncestry(data.GenotypesRefAlt):
                     record.samples[sample]["SAMPLE"] = tuple(
                         self.valid_labels[samp_idx, var_idx, :]
                     )
-                # TODO: add proper phase info
-                record.samples[sample].phased = True
+                # add proper phasing info
+                if phased:
+                    record.samples[sample].phased = True
+                else:
+                    record.samples[sample].phased = self.data[samp_idx, var_idx, 2]
             # write the record to a file
             vcf.write(record)
         vcf.close()
@@ -597,7 +601,7 @@ def transform_haps(
         if ancestry and not bps_file.exists():
             gt = GenotypesAncestry(fname=genotypes, log=log)
         else:
-            gt = data.GenotypesRefAlt(fname=genotypes, log=log)
+            gt = data.GenotypesVCF(fname=genotypes, log=log)
     # gt._prephased = True
     gt.read(region=region, samples=samples, variants=variants)
     gt.check_missing(discard_also=discard_missing)
@@ -631,7 +635,7 @@ def transform_haps(
         bps = data.Breakpoints(fname=bps_file, log=log)
         bps.read(samples=set(gt.samples))
         bps.encode()
-        # convert the GenotypesRefAlt object to a GenotypesAncestry object
+        # convert the GenotypesVCF object to a GenotypesAncestry object
         # TODO: figure out a better solution for this
         # this is just a temp hack to get output from simgenotype to load a bit faster
         gta = GenotypesAncestry(fname=None, log=log)
@@ -647,7 +651,7 @@ def transform_haps(
         hp_gt = data.GenotypesPLINK(fname=output, log=log, chunk_size=chunk_size)
     else:
         out_file_type = "VCF/BCF"
-        hp_gt = data.GenotypesRefAlt(fname=output, log=log)
+        hp_gt = data.GenotypesVCF(fname=output, log=log)
     log.info("Transforming genotypes via haplotypes")
     hp.transform(gt, hp_gt)
 
